@@ -1,7 +1,7 @@
 =begin
   gettext.rb - GetText module
 
-  Copyright (C) 2001-2007  Masao Mutoh
+  Copyright (C) 2001-2008  Masao Mutoh
   Copyright (C) 2001-2003  Masahiro Sakai
 
       Masao Mutoh       <mutoh@highway.ne.jp>
@@ -10,13 +10,13 @@
   You may redistribute it and/or modify it under the same
   license terms as Ruby.
 
-  $Id: gettext.rb,v 1.27 2007/07/05 05:29:28 mutoh Exp $
+  $Id: gettext.rb,v 1.46 2008/09/13 18:23:55 mutoh Exp $
 =end
 
 require 'rbconfig'
 require 'gettext/version'
 require 'gettext/mo'
-require 'gettext/locale'
+require 'locale'
 require 'gettext/textdomainmanager'
 require 'gettext/string'
 
@@ -29,7 +29,10 @@ module GetText
     mod.extend self
   end
 
-  @@__cached = true
+  # Max cached object_ids includes dead objects.
+  CACHE_BOUND_TARGET_MAX_SIZE = 50000 #:nodoc:
+
+  @@__cached = ! $DEBUG
 
   # Set the value whether cache messages or not. 
   # true to cache messages, otherwise false.
@@ -45,14 +48,17 @@ module GetText
   def cached?
     @@__cached
   end
-
+  
   # Clear the cached messages.
   def clear_cache
     @@__cache_msgids = {}
     @@__cache_nmsgids = {}
+    @@__cache_target_classes = {}
+    @@__cache_bound_target = {}
+    @@__cache_bound_targets = {}
   end
 
-  @@__textdomainmanagers = Hash.new
+  @@__textdomainmanagers = {}
 
   # call-seq:
   # bindtextdomain(domainname, options = {})
@@ -85,15 +91,16 @@ module GetText
       opt = options
     end
     opt[:locale] = opt[:locale] ? Locale::Object.new(opt[:locale]) : Locale.get
-    opt[:charset] = output_charset if output_charset
-    locale.charset = opt[:charset] if opt[:charset]
+    opt[:charset] = TextDomainManager.output_charset if TextDomainManager.output_charset
+    opt[:locale].charset = opt[:charset] if opt[:charset]
     Locale.set_current(opt[:locale])
-    manager = @@__textdomainmanagers[bound_target]
+    target_key = bound_target
+    manager = @@__textdomainmanagers[target_key]
     if manager
       manager.set_locale(opt[:locale]) 
     else
-      manager = TextDomainManager.new(bound_target, opt[:locale])
-      @@__textdomainmanagers[bound_target] = manager
+      manager = TextDomainManager.new(target_key, opt[:locale])
+      @@__textdomainmanagers[target_key] = manager
     end
     manager.add_textdomain(domainname, opt)
     manager
@@ -121,10 +128,11 @@ module GetText
   def textdomain(domainname)
     domain = TextDomainManager.textdomain(domainname)
     raise NoboundTextDomainError, "#{domainname} is not bound." unless domain
-    manager = @@__textdomainmanagers[bound_target]
+    target_key = bound_target
+    manager = @@__textdomainmanagers[target_key]
     unless manager
-      manager = TextDomainManager.new(bound_target, Locale.get)
-      @@__textdomainmanagers[bound_target] = manager
+      manager = TextDomainManager.new(target_key, Locale.get)
+      @@__textdomainmanagers[target_key] = manager
     end
     manager.set_locale(Locale.get)
     manager.add_textdomain(domainname)
@@ -148,7 +156,7 @@ module GetText
   # * klass: a class/module to find. Default is the class of self.
   # * ignore_targets: Ignore tragets.
   # * Returns: a bound GetText::TextDomain or nil.
-  def each_textdomain(klass = bound_target, ignore_targets = []) #:nodoc:
+  def each_textdomain(klass = self, ignore_targets = []) #:nodoc:
     bound_targets(klass).each do |target|
       unless ignore_targets.include? target
 	manager = @@__textdomainmanagers[target]
@@ -162,23 +170,74 @@ module GetText
     self
   end
 
-  def bound_target # :nodoc:
-    if self.kind_of? Class or self.kind_of? Module
-      self
-    else
-      self.class
+  @@__cache_target_classes = {}
+
+  def find_targets(klass)  #:nodoc:
+    unless (klass.kind_of? Class or klass.kind_of? Module)
+      klass = klass.class
     end
+    id = klass.object_id
+    if cached?
+      return @@__cache_target_classes[id] if @@__cache_target_classes[id]
+    end
+
+    ret = []
+    ary = klass.name.split(/::/)
+    while(v = ary.shift)
+      if ret.size == 0
+        if v.kind_of? Class
+          target = v
+        else
+          target = eval(v)
+          return [Object] unless (target = eval(v)) # For anonymous module
+        end
+      else
+        target = ret[0].const_get(v)
+      end
+      ret.unshift(target) if target
+    end
+    @@__cache_target_classes[id] = ret.size > 0 ? ret : [klass]
   end
 
+  @@__cache_bound_target = {}
+
+  def bound_target(klass = self) # :nodoc:
+    ret = (klass.kind_of? Module) ? klass : klass.class
+    id = ret.object_id
+    if cached?
+     tgt = @@__cache_bound_target[id]
+     return tgt if tgt
+    end
+
+    if ret.name =~ /^\#<|^$/ or ret == GetText
+       #GC for dead object_ids.
+       ret = Object
+       if @@__cache_bound_target.size > CACHE_BOUND_TARGET_MAX_SIZE
+          @@__cache_bound_target.clear
+       end
+    end
+    @@__cache_bound_target[id] = ret
+    ret
+  end
+
+  @@__cache_bound_targets = {}
+
   def bound_targets(klass)  # :nodoc:
+    id = klass.object_id
+    if cached?
+      if @@__cache_bound_targets[id]
+        return @@__cache_bound_targets[id]
+      end
+    end
     ret = []
+    klass = bound_target(klass)
     ary = klass.name.split(/::/)
     while(v = ary.shift)
       ret.unshift(((ret.size == 0) ? eval(v) : ret[0].const_get(v)))
     end
-    ((ret + klass.ancestors + [GetText]) & @@__textdomainmanagers.keys).uniq
+    @@__cache_bound_targets[id] = ((ret + klass.ancestors + [Object]) & @@__textdomainmanagers.keys).uniq
   end
-
+ 
   @@__cache_msgids = {}
 
   # call-seq:
@@ -186,6 +245,19 @@ module GetText
   #   _(msgid)
   #
   # Translates msgid and return the message.
+  # This doesn't make a copy of the message. 
+  #
+  # You need to use String#dup if you want to modify the return value 
+  # with destructive functions. 
+  #
+  # (e.g.1) _("Hello ").dup << "world"
+  # 
+  # But e.g.1 should be rewrite to:
+  #
+  # (e.g.2) _("Hello %{val}") % {:val => "world"}
+  #
+  # Because the translator may want to change the position of "world".
+  #
   # * msgid: the message id.
   # * Returns: localized text by msgid. If there are not binded mo-file, it will return msgid.
   def gettext(msgid)
@@ -206,16 +278,23 @@ module GetText
   # See: http://www.gnu.org/software/gettext/manual/html_mono/gettext.html#SEC151
   def sgettext(msgid, div = '|')
     cached_key = [bound_target, Locale.current, msgid]
-    if (@@__cached or ! $DEBUG)
-      if @@__cache_msgids.keys.include?(cached_key)
+    if cached?
+      if @@__cache_msgids[cached_key]
         return @@__cache_msgids[cached_key]
       end
     end
     msg = nil
-    each_textdomain {|textdomain|
-      msg = textdomain.gettext(msgid)
+
+    # Use "for"(not "each") to support JRuby 1.1.0.
+    for target in bound_targets(self)
+      manager = @@__textdomainmanagers[target]
+      for textdomain in manager.textdomains
+        msg = textdomain[1].gettext(msgid)
+        break if msg
+      end
       break if msg
-    }
+    end
+
     msg ||= msgid
     if div and msg == msgid
       if index = msg.rindex(div)
@@ -225,6 +304,23 @@ module GetText
     @@__cache_msgids[cached_key] = msg
   end
 
+  # call-seq:
+  #   pgettext(msgctxt, msgid)
+  #   p_(msgctxt, msgid)
+  #
+  # Translates msgid with msgctxt. This methods is similer with s_().
+  #  e.g.) p_("File", "New")   == s_("File|New")
+  #        p_("File", "Open")  == s_("File|Open")
+  #
+  # * msgctxt: the message context.
+  # * msgid: the message id.
+  # * Returns: the localized text by msgid. If there are no localized text, 
+  #   it returns msgid.
+  # See: http://www.gnu.org/software/autoconf/manual/gettext/Contexts.html
+  def pgettext(msgctxt, msgid)
+    sgettext(msgctxt + "\004" + msgid, "\004")
+  end
+
   @@__cache_nmsgids = {}
 
   # call-seq:
@@ -245,6 +341,42 @@ module GetText
     nsgettext(arg1, arg2, arg3, nil)
   end
 
+  # call-seq:
+  #   npgettext(msgctxt, msgid, msgid_plural, n)
+  #   npgettext(msgctxt, msgids, n)  # msgids = [msgid, msgid_plural]
+  #   np_(msgctxt, msgid, msgid_plural, n)
+  #   np_(msgctxt, msgids, n)  # msgids = [msgid, msgid_plural]
+  #
+  # The npgettext is similar to the nsgettext function.
+  #   e.g.) np_("Special", "An apple", "%{num} Apples", num) == ns_("Special|An apple", "%{num} Apples", num)
+  # * msgctxt: the message context.
+  # * msgid: the singular form.
+  # * msgid_plural: the plural form.
+  # * n: a number used to determine the plural form.
+  # * Returns: the localized text which key is msgid_plural if n is plural(follow plural-rule) or msgid.
+  #   "plural-rule" is defined in po-file.
+  def npgettext(msgctxt, arg1, arg2 = nil, arg3 = nil)
+    if arg1.kind_of?(Array)
+      msgid = arg1[0]
+      msgid_ctxt = "#{msgctxt}\004#{msgid}"
+      msgid_plural = arg1[1]
+      opt1 = arg2
+      opt2 = arg3
+    else
+      msgid = arg1
+      msgid_ctxt = "#{msgctxt}\004#{msgid}"
+      msgid_plural = arg2
+      opt1 = arg3
+      opt2 = nil
+    end
+    ret = nsgettext(msgid_ctxt, msgid_plural, opt1, opt2)
+    
+    if ret == msgid_ctxt
+      ret = msgid
+    end
+    ret
+  end
+
   # This function does nothing. But it is required in order to recognize the msgid by rgettext.
   # * msgid: the message id.
   # * Returns: msgid.
@@ -294,16 +426,21 @@ module GetText
 
     cached_key = [bound_target, Locale.current, msgid + "\000" + msgid_plural]
     msgs = nil
-    if (@@__cached or ! $DEBUG)
-      if @@__cache_nmsgids.keys.include?(cached_key)
+    if @@__cached
+      if @@__cache_nmsgids.has_key?(cached_key)
         msgs = @@__cache_nmsgids[cached_key]  # [msgstr, cond_as_string]
       end
     end
     unless msgs
-      each_textdomain {|textdomain|
-        msgs = textdomain.ngettext_data(msgid, msgid_plural)
+      # Use "for"(not "each") to support JRuby 1.1.0.
+      for target in bound_targets(self)
+        manager = @@__textdomainmanagers[target]
+        for textdomain in manager.textdomains
+          msgs = textdomain[1].ngettext_data(msgid, msgid_plural)
+          break if msgs
+        end
         break if msgs
-      }
+      end
       msgs = [[msgid, msgid_plural], "n != 1"] unless msgs
       @@__cache_nmsgids[cached_key] = msgs
     end
@@ -338,7 +475,7 @@ module GetText
       else
 	ret = Locale::Object.new(locale.to_s)
       end
-      ret.charset = output_charset if output_charset
+      ret.charset = TextDomainManager.output_charset if TextDomainManager.output_charset
       Locale.set(ret)
     else
       Locale.set(nil)
@@ -347,11 +484,11 @@ module GetText
     if this_target_only
       manager = @@__textdomainmanagers[bound_target]
       if manager
-	manager.set_locale(ret)
+	manager.set_locale(ret, ! cached?)
       end
     else
       each_textdomain {|textdomain|
-	textdomain.set_locale(ret)
+        textdomain.set_locale(ret, ! cached?)
       }
     end
     self
@@ -373,10 +510,10 @@ module GetText
     else
       ret = Locale.default
     end
-    ret.charset = output_charset if output_charset
+    ret.charset = TextDomainManager.output_charset if TextDomainManager.output_charset
     Locale.set_current(ret)
     TextDomainManager.each_all {|textdomain|
-      textdomain.set_locale(ret)
+      textdomain.set_locale(ret, ! cached?)
     }
     self
   end
@@ -412,7 +549,7 @@ module GetText
   # Gets the current output_charset which is set using GetText.set_output_charset.
   # * Returns: output_charset.
   def output_charset
-    TextDomainManager.output_charset
+    TextDomainManager.output_charset || locale.charset
   end
 
   # Gets the current locale.
@@ -421,17 +558,6 @@ module GetText
     Locale.current
   end
 
-  # Deprecated. Now this function do nothing. Use GetText.output_charset= instead.
-  def set_charset(cs) #:nodoc:
-    $stderr.puts "Deprecated. Now this function do nothing. Use GetText.output_charset= instead." if $DEBUG
-    self
-  end
-
-  def charset=(cs) #:nodoc:
-    set_charset(cs)
-    cs
-  end
-  
   # Add default locale path.
   # * path: a new locale path. (e.g.) "/usr/share/locale/%{locale}/LC_MESSAGES/%{name}.mo"
   #   ('locale' => "ja_JP", 'name' => "textdomain")
@@ -445,7 +571,6 @@ module GetText
   #   * :with_messages - show informations with messages of the current mo file. Default is false.
   #   * :out - An output target. Default is STDOUT.
   #   * :with_paths - show the load paths for mo-files.
-  # * Returns: localized text by msgid. If there are not binded mo-file, it will return msgid.
   def current_textdomain_info(options = {})
     opts = {:with_messages => false, :with_paths => false, :out => STDOUT}.merge(options)
     ret = nil
@@ -468,14 +593,29 @@ module GetText
     }
   end
 
+  # for testing.
+  def remove_all_textdomains
+    clear_cache
+    @@__textdomainmanagers = {}
+  end
+
   alias :setlocale :locale= #:nodoc:
   alias :_ :gettext   #:nodoc:
   alias :n_ :ngettext #:nodoc:
   alias :s_ :sgettext #:nodoc:
   alias :ns_ :nsgettext #:nodoc:
+  alias :np_ :npgettext #:nodoc:
 
-  module_function :bindtextdomain, :textdomain, :each_textdomain, :cached=, :cached?, :clear_cache,
-    :N_, :gettext, :_, :ngettext, :n_, :sgettext, :s_, :nsgettext, :ns_, :bound_target, :bound_targets,
-    :setlocale, :set_locale, :locale=, :set_locale_all, :locale, :charset=, :set_charset,
-    :set_output_charset, :output_charset=, :output_charset, :current_textdomain_info
+unless defined? "XX"
+  # This is the workaround to conflict p_ methods with the xx("double x") library.
+  # http://rubyforge.org/projects/codeforpeople/
+  alias :p_ :pgettext #:nodoc:
+  module_function :p_
+end
+
+  module_function :bindtextdomain, :textdomain, :each_textdomain, :cached=, :cached?, :clear_cache, 
+  :N_, :gettext, :_, :ngettext, :n_, :sgettext, :s_, :nsgettext, :ns_, :pgettext, :npgettext, :np_,
+  :bound_target, :bound_targets, :find_targets,
+  :setlocale, :set_locale, :locale=, :set_locale_all, :locale, 
+  :set_output_charset, :output_charset=, :output_charset, :current_textdomain_info, :remove_all_textdomains 
 end
