@@ -6,7 +6,7 @@
   You may redistribute it and/or modify it under the same
   license terms as Ruby.
 
-  $Id: rails.rb,v 1.72 2008/01/29 16:30:29 mutoh Exp $
+  $Id: rails.rb,v 1.81 2008/09/13 18:23:56 mutoh Exp $
 =end
 
 require 'gettext/cgi'
@@ -228,6 +228,7 @@ module ActionController #:nodoc:
       textdomain_to(ActionView::Base, domainname) if defined? ActionView::Base
       textdomain_to(ApplicationHelper, domainname) if defined? ApplicationHelper
       textdomain_to(ActionMailer::Base, domainname) if defined? ActionMailer::Base
+      textdomain_to(ActionView::Helpers, domainname) if defined? ActionView::Helpers
     end
 
     # Gets the textdomain name and path of this controller which is set 
@@ -270,21 +271,52 @@ end
 
 module ActionView #:nodoc:
   class Base #:nodoc:
-    alias render_file_without_locale render_file #:nodoc:
-    # This provides to find localized template files such as foo_ja.rhtml, foo_ja_JP.rhtml
-    # instead of foo.rhtml. If the file isn't found, foo.rhtml is used.
-    def render_file(template_path, use_full_path = true, local_assigns = {})
+    # This provides to find localized template files such as foo_ja.html.erb, foo_ja_JP.html.erb
+    # instead of foo.html.erb. If the file isn't found, foo.html.erb is used.
+    def render_file_with_gettext(template_path, use_full_path = true, local_assigns = {})
       locale = GetText.locale
-      [locale.to_general, locale.to_s, locale.language, Locale.default.language].uniq.each do |v|
-	localized_path = "#{template_path}_#{v}"
-	return render_file_without_locale(localized_path, use_full_path, local_assigns) if file_exists? localized_path
+      begin
+        [locale.to_general, locale.to_s, locale.language, Locale.default.language].uniq.each do |v|
+          localized_path = "#{template_path}_#{v}"
+          if (respond_to? :file_exists?) ? file_exists?(localized_path) : (finder.file_exists? localized_path)
+            return render_file_without_gettext(localized_path, use_full_path, local_assigns)
+          end
+        end
+      rescue Exception
+        logger.debug($!)
+        logger.debug($!.backtrace)
       end
-      render_file_without_locale(template_path, use_full_path, local_assigns)
+      render_file_without_gettext(template_path, use_full_path, local_assigns)
     end
-    
+    alias_method_chain :render_file, :gettext
   end
 
+if defined? TemplateFinder
+  class TemplateFinder
+    def pick_template_with_gettext(template_path, extension)
+      locale = GetText.locale
+      [locale.to_general, locale.to_s, locale.language, Locale.default.language].uniq.each do |v|
+        file_name = "#{template_path}_#{v}.#{extension}"
+        base_path = find_base_path_for(file_name)
+        return "#{base_path}/#{file_name}" unless base_path.blank?
+      end
+      pick_template_without_gettext(template_path, extension)
+    end
+    alias_method_chain :pick_template, :gettext
+  end
+end
+
   module Helpers  #:nodoc:
+    class FormBuilder
+      include GetText
+      
+      def label_with_gettext(method, text = nil, options = {})
+        text ||= s_("#{@object.class}|#{method.to_s.humanize}")
+        @template.label(@object_name, method, text, options.merge(:object => @object))
+      end
+      alias_method_chain :label, :gettext
+    end
+
     module ActiveRecordHelper #:nodoc: all
       module L10n
 	# Separate namespace for textdomain
@@ -402,18 +434,19 @@ module ActionView #:nodoc:
       # This is FAKE constant. The messages are found by rgettext as the msgid. 
       MESSAGESS = [N_('less than 5 seconds'), N_('less than 10 seconds'), N_('less than 20 seconds'),
                    N_('half a minute'), N_('less than a minute'), N_('about 1 month'), 
-                   N_('about 1 year'), N_('over 2 years')]
+                   N_('about 1 year')]
       NMINUTES = [/^(\d+) minutes?$/, Nn_('1 minute', '%{num} minutes')]
       NHOURS   = [/^about (\d+) hours?$/, Nn_('about 1 hour', 'about %{num} hours')]
       NDAYS    = [/^(\d+) days?$/, Nn_('1 day', '%{num} days')]
       NMONTHS  = [/^(\d+) months?$/, Nn_('1 month', '%{num} months')]
+      NYEARS  = [/^over (\d+) years?$/, Nn_('over 1 year', 'over %{num} years')]
 
       def distance_of_time_in_words(from_time, to_time = 0, include_seconds = false)
 	textdomain("rails")
 
 	msg = distance_of_time_in_words_without_locale(from_time, to_time, include_seconds)
 	match = false
-	[NMINUTES, NHOURS, NDAYS, NMONTHS].each do |regexp, nn|
+	[NMINUTES, NHOURS, NDAYS, NMONTHS, NYEARS].each do |regexp, nn|
 	  if regexp =~ msg
 	    match = true
 	    msg = n_(nn, $1.to_i) % {:num => $1}
@@ -478,16 +511,17 @@ module ActionController #:nodoc: all
       def expire_fragment_with_gettext(name, options = nil)
         return unless perform_caching
 
-        key = fragment_cache_key_without_gettext(name)
+        fc_store = (respond_to? :cache_store) ? cache_store : fragment_cache_store
+        key = name.is_a?(Regexp) ? name : fragment_cache_key_without_gettext(name)
         if key.is_a?(Regexp)
           self.class.benchmark "Expired fragments matching: #{key.source}" do
-            fragment_cache_store.delete_matched(key, options)
+            fc_store.delete_matched(key, options)
           end
         else
           key = key.gsub(/:/, ".")
           self.class.benchmark "Expired fragment: #{key}, lang = #{available_locales.inspect}" do
             available_locales.each do |lang|
-              fragment_cache_store.delete("#{key}_#{lang}", options)
+              fc_store.delete("#{key}_#{lang}", options)
             end
           end
         end
