@@ -11,11 +11,13 @@ require 'active_record/fixtures'
 require 'fixtures/topic'
 require 'fixtures/reply'
 require 'fixtures/developer'
+require 'fixtures/wizard'
+require 'fixtures/inept_wizard'
 require 'logger'
 
-version = /activerecord-([^\/]+)/.match($:.join)[1]
+AR_TEST_VERSION = /activerecord-([^\/]+)/.match($:.join)[1]
 
-if version > "2.0.0"
+if AR_TEST_VERSION > "2.0.0"
   #ticket 6657 on dev.rubyonrails.org require this but it becames removed(?)
   AR_6657 = true
 else
@@ -67,6 +69,12 @@ class Topic
   end
 end
 
+class ProtectedPerson < ActiveRecord::Base
+  set_table_name 'people'
+  attr_accessor :addon
+  attr_protected :first_name
+end
+
 class MyModel
   attr_accessor :title
   def save; end
@@ -83,16 +91,30 @@ class MyModel
   validates_presence_of :title
 end
 
+class Conjurer < IneptWizard
+end
+
+class Thaumaturgist < IneptWizard
+end
+
 class ValidationsTest < Test::Unit::TestCase
   fixtures :topics, :developers
 
   def setup
     bindtextdomain_to(Topic, "active_record")
-    Topic.write_inheritable_attribute(:validate, nil)
-    Topic.write_inheritable_attribute(:validate_on_create, nil)
-    Topic.write_inheritable_attribute(:validate_on_update, nil)
+
+    if AR_TEST_VERSION < "2.1.0"
+      Topic.write_inheritable_attribute(:validate, nil)
+      Topic.write_inheritable_attribute(:validate_on_create, nil)
+      Topic.write_inheritable_attribute(:validate_on_update, nil)
+    else
+      Topic.instance_variable_set("@validate_callbacks", ActiveSupport::Callbacks::CallbackChain.new)
+      Topic.instance_variable_set("@validate_on_create_callbacks", ActiveSupport::Callbacks::CallbackChain.new)
+      Topic.instance_variable_set("@validate_on_update_callbacks", ActiveSupport::Callbacks::CallbackChain.new)
+    end
   end
 
+
   def test_single_attr_validation_and_error_msg
     GetText.set_locale_all "ja_JP.UTF-8"
     r = Reply.new
@@ -210,6 +232,77 @@ class ValidationsTest < Test::Unit::TestCase
     end
   end
 
+  def test_exception_on_create_bang_many
+    assert_raises(ActiveRecord::RecordInvalid) do
+      Reply.create!([ { "title" => "OK" }, { "title" => "Wrong Create" }])
+    end
+
+    GetText.set_locale_all "ja_JP.UTF-8"
+    begin
+      Reply.create!([ { "title" => "OK" }, { "title" => "Wrong Create" }])
+    rescue ActiveRecord::RecordInvalid => invalid
+      assert_equal "入力値が正しくありません。: Reply内容 空です。", invalid.message
+    end
+    GetText.set_locale_all "en"
+    begin
+      Reply.create!([ { "title" => "OK" }, { "title" => "Wrong Create" }])
+    rescue ActiveRecord::RecordInvalid => invalid
+      assert_equal "Validation failed: Content Empty", invalid.message
+    end
+  end
+
+ 
+  def test_exception_on_create_bang_with_block
+    assert_raises(ActiveRecord::RecordInvalid) do
+      Reply.create!({ "title" => "OK" }) do |r|
+        r.content = nil
+      end
+    end
+
+    GetText.set_locale_all "ja_JP.UTF-8"
+    begin
+      Reply.create!({ "title" => "OK" }) do |r|
+        r.content = nil
+      end
+    rescue ActiveRecord::RecordInvalid => invalid
+      assert_equal "入力値が正しくありません。: Reply内容 空です。", invalid.message
+    end
+
+    GetText.set_locale_all "en"
+    begin
+      Reply.create!({ "title" => "OK" }) do |r|
+        r.content = nil
+      end
+    rescue ActiveRecord::RecordInvalid => invalid
+      assert_equal "Validation failed: Content Empty", invalid.message
+    end
+  end
+
+  def test_exception_on_create_bang_many_with_block
+    assert_raises(ActiveRecord::RecordInvalid) do
+      Reply.create!([{ "title" => "OK" }, { "title" => "Wrong Create" }]) do |r|
+        r.content = nil
+      end
+    end
+    GetText.set_locale_all "ja_JP.UTF-8"
+    begin
+      Reply.create!([{ "title" => "OK" }, { "title" => "Wrong Create" }]) do |r|
+        r.content = nil
+      end
+    rescue ActiveRecord::RecordInvalid => invalid
+      assert_equal "入力値が正しくありません。: Reply内容 空です。", invalid.message
+    end
+
+    GetText.set_locale_all "en"
+    begin
+      Reply.create!([{ "title" => "OK" }, { "title" => "Wrong Create" }]) do |r|
+        r.content = nil
+      end
+    rescue ActiveRecord::RecordInvalid => invalid
+      assert_equal "Validation failed: Content Empty", invalid.message
+    end
+  end
+  
   def test_scoped_create_without_attributes
     Reply.with_scope(:create => {}) do
       assert_raises(ActiveRecord::RecordInvalid) { Reply.create! }
@@ -234,7 +327,6 @@ class ValidationsTest < Test::Unit::TestCase
     end
   end
 
-
   def test_single_error_per_attr_iteration
     GetText.set_locale_all "ja_JP.UTF-8"
     r = Reply.new
@@ -350,6 +442,16 @@ class ValidationsTest < Test::Unit::TestCase
     perform = false
   end
 
+=begin 
+  #Don't need this
+  def test_no_title_confirmation
+  end
+  def test_title_confirmation
+  end
+  def test_terms_of_service_agreement_no_acceptance
+  end
+=end
+
   def test_errors_on_boundary_breaking
     GetText.set_locale_all "ja_JP.UTF-8"  
     developer = Developer.new("name" => "xs")
@@ -467,6 +569,14 @@ class ValidationsTest < Test::Unit::TestCase
     assert t.save
   end
 
+=begin
+  #Don't need this
+  def test_validates_acceptance_of_as_database_column
+  end
+  def test_validates_acceptance_of_with_non_existant_table
+  end
+=end
+
   def test_validate_presences
     Topic.validates_presence_of(:title, :content)
 
@@ -539,7 +649,80 @@ class ValidationsTest < Test::Unit::TestCase
     assert_equal "Title has already been taken", t2.errors.on(:title)
   end
 
-  def test_validate_format_fn
+=begin
+  #Don't need this
+  def test_validate_uniqueness_with_scope
+  end
+  def test_validate_uniqueness_scoped_to_defining_class
+  end
+  def test_validate_uniqueness_with_scope_array
+  end
+  def test_validate_case_insensitive_uniqueness
+  end
+  def test_validate_case_sensitive_uniqueness
+  end
+=end
+
+   def test_validate_straight_inheritance_uniqueness
+    GetText.set_locale_all "ja_JP.UTF-8"  
+    w1 = IneptWizard.create(:name => "I18nRincewind", :city => "I18nAnkh-Morpork")
+    assert w1.valid?, "Saving w1"
+
+    # Should use validation from base class (which is abstract)
+    w2 = IneptWizard.new(:name => "I18nRincewind", :city => "I18nQuirm")
+    assert !w2.valid?, "w2 shouldn't be valid"
+    assert w2.errors.on(:name), "Should have errors for name"
+    assert_equal "不器用な魔術師名はすでに存在します。", w2.errors.on(:name), "Should have uniqueness message for name"
+
+    w3 = Conjurer.new(:name => "I18nRincewind", :city => "I18nQuirm")
+    assert !w3.valid?, "w3 shouldn't be valid"
+    assert w3.errors.on(:name), "Should have errors for name"
+    assert_equal "手品師名はすでに存在します。", w3.errors.on(:name), "Should have uniqueness message for name"
+
+    w4 = Conjurer.create(:name => "I18nThe Amazing Bonko", :city => "I18nQuirm")
+    assert w4.valid?, "Saving w4"
+
+    w5 = Thaumaturgist.new(:name => "I18nThe Amazing Bonko", :city => "I18nLancre")
+    assert !w5.valid?, "w5 shouldn't be valid"
+    assert w5.errors.on(:name), "Should have errors for name"
+    assert_equal "奇術師名はすでに存在します。", w5.errors.on(:name), "Should have uniqueness message for name"
+
+    w6 = Thaumaturgist.new(:name => "I18nMustrum Ridcully", :city => "I18nQuirm")
+    assert !w6.valid?, "w6 shouldn't be valid"
+    assert w6.errors.on(:city), "Should have errors for city"
+    assert_equal "奇術師町はすでに存在します。", w6.errors.on(:city), "Should have uniqueness message for city"
+
+    GetText.set_locale_all "en"  
+    w1 = IneptWizard.create(:name => "Rincewind", :city => "Ankh-Morpork")
+    assert w1.valid?, "Saving w1"
+
+    # Should use validation from base class (which is abstract)
+    w2 = IneptWizard.new(:name => "Rincewind", :city => "Quirm")
+    assert !w2.valid?, "w2 shouldn't be valid"
+    assert w2.errors.on(:name), "Should have errors for name"
+    assert_equal "Name has already been taken", w2.errors.on(:name), "Should have uniqueness message for name"
+
+    w3 = Conjurer.new(:name => "Rincewind", :city => "Quirm")
+    assert !w3.valid?, "w3 shouldn't be valid"
+    assert w3.errors.on(:name), "Should have errors for name"
+    assert_equal "Name has already been taken", w3.errors.on(:name), "Should have uniqueness message for name"
+
+    w4 = Conjurer.create(:name => "The Amazing Bonko", :city => "Quirm")
+    assert w4.valid?, "Saving w4"
+
+    w5 = Thaumaturgist.new(:name => "The Amazing Bonko", :city => "Lancre")
+    assert !w5.valid?, "w5 shouldn't be valid"
+    assert w5.errors.on(:name), "Should have errors for name"
+    assert_equal "Name has already been taken", w5.errors.on(:name), "Should have uniqueness message for name"
+
+    w6 = Thaumaturgist.new(:name => "Mustrum Ridcully", :city => "Quirm")
+    assert !w6.valid?, "w6 shouldn't be valid"
+    assert w6.errors.on(:city), "Should have errors for city"
+    assert_equal "City has already been taken", w6.errors.on(:city), "Should have uniqueness message for city"
+   end
+
+
+   def test_validate_format_fn
     Topic.validates_format_of(:title, :content, :with => /^Validation\smacros \w+!$/, :message => N_("%{fn} is bad data"))
 
     GetText.set_locale_all "ja_JP.UTF-8"  
@@ -588,6 +771,63 @@ class ValidationsTest < Test::Unit::TestCase
     assert_raise(ArgumentError) { Topic.validates_format_of(:title, :content) }
   end
 
+=begin
+  # Don't need this
+  def test_validate_format_with_allow_blank
+  end
+=end
+  def test_validate_format_numeric_fn
+    Topic.validates_format_of(:title, :content, :with => /^[1-9][0-9]*$/, :message => N_("%{fn} is bad data"))
+
+    GetText.set_locale_all "ja_JP.UTF-8"
+    t = Topic.create("title" => "72x", "content" => "6789")
+    assert !t.valid?, "Shouldn't be valid"
+    assert !t.save, "Shouldn't save because it's invalid"
+    assert_equal "タイトルは悪いデータです。", t.errors.on(:title)
+    assert_nil t.errors.on(:content)
+    
+    t.title = "-11"
+    assert !t.valid?, "Shouldn't be valid"
+
+    t.title = "03"
+    assert !t.valid?, "Shouldn't be valid"
+
+    t.title = "z44"
+    assert !t.valid?, "Shouldn't be valid"
+
+    t.title = "5v7"
+    assert !t.valid?, "Shouldn't be valid"
+
+    t.title = "1"
+
+    assert t.save
+    assert_nil t.errors.on(:title)
+
+    GetText.set_locale_all "en"
+    t = Topic.create("title" => "72x", "content" => "6789")
+    assert !t.valid?, "Shouldn't be valid"
+    assert !t.save, "Shouldn't save because it's invalid"
+    assert_equal "Title is bad data", t.errors.on(:title)
+    assert_nil t.errors.on(:content)
+
+    t.title = "-11"
+    assert !t.valid?, "Shouldn't be valid"
+
+    t.title = "03"
+    assert !t.valid?, "Shouldn't be valid"
+
+    t.title = "z44"
+    assert !t.valid?, "Shouldn't be valid"
+
+    t.title = "5v7"
+    assert !t.valid?, "Shouldn't be valid"
+
+    t.title = "1"
+
+    assert t.save
+    assert_nil t.errors.on(:title)
+  end
+
   def test_validate_format_numeric
     Topic.validates_format_of(:title, :content, :with => /^[1-9][0-9]*$/, :message => N_("is bad data"))
 
@@ -639,7 +879,35 @@ class ValidationsTest < Test::Unit::TestCase
     assert t.save
     assert_nil t.errors.on(:title)
   end
-  
+
+  def test_validate_format_with_formatted_message_fn
+    Topic.validates_format_of(:title, :with => /^Valid Title$/, :message => N_("%{fn} can not be %{val}"))
+
+    GetText.set_locale_all "ja_JP.UTF-8"
+    t = Topic.create(:title => 'Invalid title')
+    assert_equal "Invalid titleはタイトルではありません。", t.errors.full_messages[0]
+    assert_equal "Invalid titleはタイトルではありません。", t.errors.on(:title)
+
+    GetText.set_locale_all "en"
+    t = Topic.create(:title => 'Invalid title')
+    assert_equal "Title can not be Invalid title", t.errors.full_messages[0]
+    assert_equal "Title can not be Invalid title", t.errors.on(:title)
+  end
+ 
+  def test_validate_format_with_formatted_message
+    Topic.validates_format_of(:title, :with => /^Valid Title$/, :message => N_("can not be %{val}"))
+
+    GetText.set_locale_all "ja_JP.UTF-8"
+    t = Topic.create(:title => 'Invalid title')
+    assert_equal "タイトル はInvalid titleではありません。", t.errors.full_messages[0]
+    assert_equal "はInvalid titleではありません。", t.errors.on(:title)
+
+    GetText.set_locale_all "en"
+    t = Topic.create(:title => 'Invalid title')
+    assert_equal "Title can not be Invalid title", t.errors.full_messages[0]
+    assert_equal "can not be Invalid title", t.errors.on(:title)
+  end
+
   def test_validates_inclusion_of
     Topic.validates_inclusion_of( :title, :in => %w( a b c d e f g ) )
 
@@ -662,6 +930,113 @@ class ValidationsTest < Test::Unit::TestCase
     assert_equal "Title is not included in the list", t.errors.on(:title)
   end
 
+=begin
+  # Don't need this
+  def test_validates_inclusion_of_with_allow_nil
+  end
+  def test_numericality_with_getter_method
+  end
+  def test_validates_length_of_with_allow_nil
+  end
+  def test_validates_length_of_with_allow_blank
+  end
+=end
+
+  def test_validates_inclusion_of_with_formatted_message_fn
+    Topic.validates_inclusion_of( :title, :in => %w( a b c d e f g ), :message => N_("%{fn} option %{val} is not in the list") )
+    GetText.set_locale_all "ja_JP.UTF-8"
+
+    assert Topic.create("title" => "a", "content" => "abc").valid?
+
+    t = Topic.create("title" => "uhoh", "content" => "abc")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "オプションuhohはタイトル一覧にありません。", t.errors.full_messages[0]
+    assert_equal "オプションuhohはタイトル一覧にありません。", t.errors["title"]
+
+    GetText.set_locale_all "en"
+    assert Topic.create("title" => "a", "content" => "abc").valid?
+
+    t = Topic.create("title" => "uhoh", "content" => "abc")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "Title option uhoh is not in the list", t.errors.full_messages[0]
+    assert_equal "Title option uhoh is not in the list", t.errors["title"]
+  end
+
+  def test_validates_inclusion_of_with_formatted_message
+    Topic.validates_inclusion_of( :title, :in => %w( a b c d e f g ), :message => N_("option %{val} is not in the list") )
+    GetText.set_locale_all "ja_JP.UTF-8"
+
+    assert Topic.create("title" => "a", "content" => "abc").valid?
+
+    t = Topic.create("title" => "uhoh", "content" => "abc")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "タイトル オプションuhohは一覧にありません。", t.errors.full_messages[0]
+    assert_equal "オプションuhohは一覧にありません。", t.errors["title"]
+
+    GetText.set_locale_all "en"
+    assert Topic.create("title" => "a", "content" => "abc").valid?
+
+    t = Topic.create("title" => "uhoh", "content" => "abc")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "Title option uhoh is not in the list", t.errors.full_messages[0]
+    assert_equal "option uhoh is not in the list", t.errors["title"]
+  end
+
+=begin
+  Don't need this
+  def test_numericality_with_allow_nil_and_getter_method
+  end
+  def test_validates_exclusion_of
+  end
+=end
+  def test_validates_exclusion_of_with_formatted_message_fn
+    GetText.set_locale_all "ja_JP.UTF-8"
+    Topic.validates_exclusion_of( :title, :in => %w( abe monkey ), :message => N_("%{fn} option %{val} is restricted") )
+
+    assert Topic.create("title" => "something", "content" => "abc")
+
+    t = Topic.create("title" => "monkey")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "オプションタイトルmonkeyは制限されています。", t.errors.full_messages[0]
+    assert_equal "オプションタイトルmonkeyは制限されています。", t.errors["title"]
+
+    GetText.set_locale_all "en"
+    assert Topic.create("title" => "something", "content" => "abc")
+
+    t = Topic.create("title" => "monkey")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "Title option monkey is restricted", t.errors.full_messages[0]
+    assert_equal "Title option monkey is restricted", t.errors["title"]
+  end
+
+  def test_validates_exclusion_of_with_formatted_message
+    GetText.set_locale_all "ja_JP.UTF-8"
+    Topic.validates_exclusion_of( :title, :in => %w( abe monkey ), :message => N_("option %{val} is restricted") )
+
+    assert Topic.create("title" => "something", "content" => "abc")
+
+    t = Topic.create("title" => "monkey")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "タイトル オプションmonkeyは制限されています。", t.errors.full_messages[0]
+    assert_equal "オプションmonkeyは制限されています。", t.errors["title"]
+
+    GetText.set_locale_all "en"
+    assert Topic.create("title" => "something", "content" => "abc")
+
+    t = Topic.create("title" => "monkey")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "Title option monkey is restricted", t.errors.full_messages[0]
+    assert_equal "option monkey is restricted", t.errors["title"]
+  end
+
   def test_validates_length_of_using_minimum
     Topic.validates_length_of :title, :minimum => 5
 
@@ -709,6 +1084,12 @@ class ValidationsTest < Test::Unit::TestCase
     assert_equal "Title is too long (maximum is 5 characters)", t.errors.on("title")
   end
 
+=begin
+# Don't need this
+  def test_optionally_validates_length_of_using_minimum
+  end
+=end
+
   def test_validates_length_of_using_within
     Topic.validates_length_of(:title, :content, :within => 3..5)
 
@@ -745,8 +1126,14 @@ class ValidationsTest < Test::Unit::TestCase
     assert_equal "Content is too short (minimum is 3 characters)", t.errors.on(:content)
   end
 
+=begin
+# Don't need this
+  def test_optionally_validates_length_of_using_within
+  end
+=end
+
   def test_optionally_validates_length_of_using_within_on_create_fn
-    Topic.validates_length_of :title, :content, :within => 5..10, :on => :create, :too_long => N_("my string(%{fn}) is too long: %d")
+    Topic.validates_length_of :title, :content, :within => 5..10, :on => :create, :too_short => N_("my string(%{fn}) is too short: %d"), :too_long => N_("my string(%{fn}) is too long: %d")
 
     GetText.set_locale_all "ja_JP.UTF-8"  
     t = Topic.create("title" => "thisisnotvalid", "content" => "whatever")
@@ -764,7 +1151,7 @@ class ValidationsTest < Test::Unit::TestCase
   end
 
   def test_optionally_validates_length_of_using_within_on_create
-    Topic.validates_length_of :title, :content, :within => 5..10, :on => :create, :too_long => N_("my string is too long: %d")
+    Topic.validates_length_of :title, :content, :within => 5..10, :on => :create, :too_short => N_("my string is too short: %d"), :too_long => N_("my string is too long: %d")
 
     GetText.set_locale_all "ja_JP.UTF-8"  
     t = Topic.create("title" => "thisisnotvalid", "content" => "whatever")
@@ -782,7 +1169,7 @@ class ValidationsTest < Test::Unit::TestCase
   end
 
   def test_optionally_validates_length_of_using_within_on_update_fn
-    Topic.validates_length_of :title, :content, :within => 5..10, :on => :update, :too_short => N_("my string(%{fn}) is too short: %d")
+    Topic.validates_length_of :title, :content, :within => 5..10, :on => :update, :too_short => N_("my string(%{fn}) is too short: %d"), :too_long => N_("my string(%{fn}) is too long: %d")
     GetText.set_locale_all "ja_JP.UTF-8"  
     t = Topic.create("title" => "vali", "content" => "whatever")
     assert !t.save
@@ -808,7 +1195,7 @@ class ValidationsTest < Test::Unit::TestCase
   end
 
   def test_optionally_validates_length_of_using_within_on_update
-    Topic.validates_length_of :title, :content, :within => 5..10, :on => :update, :too_short => N_("my string is too short: %d")
+    Topic.validates_length_of :title, :content, :within => 5..10, :on => :update, :too_short => N_("my string is too short: %d"), :too_long => N_("my string is too long: %d")
     GetText.set_locale_all "ja_JP.UTF-8"  
     t = Topic.create("title" => "vali", "content" => "whatever")
     assert !t.save
@@ -858,6 +1245,14 @@ class ValidationsTest < Test::Unit::TestCase
 
   end
 
+=begin
+  # Don't need this
+  def test_optionally_validates_length_of_using_is
+  end
+  def test_validates_length_of_using_bignum
+  end
+=end
+
   def test_validates_length_with_globaly_modified_error_message_fn
     ActiveRecord::Errors.default_error_messages[:too_short] = N_('%{fn} %d dayo')
     Topic.validates_length_of :title, :minimum => 10
@@ -892,6 +1287,15 @@ class ValidationsTest < Test::Unit::TestCase
     assert_equal '10 dayo', t.errors.on('title')
   end
 
+=begin
+  def test_validates_size_of_association
+  end
+  def test_validates_size_of_association_using_within
+  end
+  def test_validates_length_of_nasty_params
+  end
+=end
+
   def test_validates_length_of_custom_errors_for_minimum_with_message_fn
     Topic.validates_length_of( :title, :minimum => 5, :message => N_("%{fn} hoo %d") )
     GetText.set_locale_all "ja_JP.UTF-8"
@@ -1101,6 +1505,29 @@ class ValidationsTest < Test::Unit::TestCase
     assert_equal "hoo 5", t.errors.on("title")
   end
 
+=begin
+  def test_validates_length_of_using_minimum_utf8
+  end
+  def test_validates_length_of_using_maximum_utf8
+  end
+  def test_validates_length_of_using_within_utf8
+  end
+  def test_optionally_validates_length_of_using_within_utf8
+  end
+  def test_optionally_validates_length_of_using_within_on_create_utf8
+  end
+  def test_optionally_validates_length_of_using_within_on_update_utf8
+  end
+  def test_validates_length_of_using_is_utf8
+  end
+  def test_validates_size_of_association_utf8
+  end
+  def test_validates_associated_many
+  end
+  def test_validates_associated_one
+  end
+=end
+
   def test_validate_block_fn
     Topic.validate { |topic| topic.errors.add("title", N_("%{fn} will never be valid")) }
     GetText.set_locale_all "ja_JP.UTF-8"
@@ -1135,6 +1562,13 @@ class ValidationsTest < Test::Unit::TestCase
     assert_equal "will never be valid", t.errors.on("title")
   end
 
+=begin
+  def test_invalid_validator
+  end
+  def test_throw_away_typing
+  end
+=end
+
   def test_validates_acceptance_of_with_custom_error_using_quotes_fn
     Developer.validates_acceptance_of :salary, :message=> N_("%{fn} contains 'single' and \"double\" quotes")
  
@@ -1569,6 +2003,155 @@ class ValidationsTest < Test::Unit::TestCase
     assert_equal "This string contains 'single' and \"double\" quotes", r.errors.on(:topic)
   end
 
+=begin
+  def test_if_validation_using_method_true
+  end
+  def test_unless_validation_using_method_true
+  end
+  def test_if_validation_using_method_false
+  end
+=end
+  def test_unless_validation_using_method_false_fn
+    # When the method returns false
+    Topic.validates_length_of( :title, :maximum=>5, :too_long=>N_("%{fn} hoo %d"), :unless => :condition_is_true_but_its_not )
+
+    GetText.set_locale_all "ja"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "タイトルふー5", t.errors.full_messages[0]
+    assert_equal "タイトルふー5", t.errors["title"]
+
+    GetText.set_locale_all "en"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "Title hoo 5", t.errors.full_messages[0]
+    assert_equal "Title hoo 5", t.errors["title"]
+  end
+
+  def test_unless_validation_using_method_false
+    # When the method returns false
+    Topic.validates_length_of( :title, :maximum=>5, :too_long=>N_("hoo %d"), :unless => :condition_is_true_but_its_not )
+
+    GetText.set_locale_all "ja"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "タイトル ふー5", t.errors.full_messages[0]
+    assert_equal "ふー5", t.errors["title"]
+
+    GetText.set_locale_all "en"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "Title hoo 5", t.errors.full_messages[0]
+    assert_equal "hoo 5", t.errors["title"]
+  end
+
+  def test_if_validation_using_string_true_fn
+    # When the evaluated string returns true
+    Topic.validates_length_of( :title, :maximum=>5, :too_long=>N_("%{fn} hoo %d"), :if => "a = 1; a == 1" )
+
+    GetText.set_locale_all "ja"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "タイトルふー5", t.errors.full_messages[0]
+    assert_equal "タイトルふー5", t.errors["title"]
+
+    GetText.set_locale_all "en"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "Title hoo 5", t.errors.full_messages[0]
+    assert_equal "Title hoo 5", t.errors["title"]
+  end
+
+  def test_if_validation_using_string_true
+    # When the evaluated string returns true
+    Topic.validates_length_of( :title, :maximum=>5, :too_long=>N_("hoo %d"), :if => "a = 1; a == 1" )
+
+    GetText.set_locale_all "ja"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "タイトル ふー5", t.errors.full_messages[0]
+    assert_equal "ふー5", t.errors["title"]
+
+    GetText.set_locale_all "en"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "Title hoo 5", t.errors.full_messages[0]
+    assert_equal "hoo 5", t.errors["title"]
+  end
+
+=begin
+  def test_unless_validation_using_string_true
+  end
+  def test_if_validation_using_string_false
+  end
+=end
+  def test_unless_validation_using_string_false
+    # When the evaluated string returns false
+    Topic.validates_length_of( :title, :maximum=>5, :too_long=>N_("hoo %d"), :unless => "false")
+
+    GetText.set_locale_all "ja"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "タイトル ふー5", t.errors.full_messages[0]
+    assert_equal "ふー5", t.errors["title"]
+
+    GetText.set_locale_all "en"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "Title hoo 5", t.errors.full_messages[0]
+    assert_equal "hoo 5", t.errors["title"]
+  end
+
+  def test_if_validation_using_block_true_fn
+    # When the block returns true
+    Topic.validates_length_of( :title, :maximum=>5, :too_long=>N_("%{fn} hoo %d"),
+      :if => Proc.new { |r| r.content.size > 4 } )
+
+    GetText.set_locale_all "ja"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "タイトルふー5", t.errors.full_messages[0]
+    assert_equal "タイトルふー5", t.errors["title"]
+
+    GetText.set_locale_all "en"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "Title hoo 5", t.errors.full_messages[0]
+    assert_equal "Title hoo 5", t.errors["title"]
+  end
+
+  def test_if_validation_using_block_true
+    # When the block returns true
+    Topic.validates_length_of( :title, :maximum=>5, :too_long=>N_("hoo %d"),
+      :if => Proc.new { |r| r.content.size > 4 } )
+
+    GetText.set_locale_all "ja"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "タイトル ふー5", t.errors.full_messages[0]
+    assert_equal "ふー5", t.errors["title"]
+
+    GetText.set_locale_all "en"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "Title hoo 5", t.errors.full_messages[0]
+    assert_equal "hoo 5", t.errors["title"]
+  end
+
   def test_conditional_validation_using_method_true_fn
     # When the method returns true
     Topic.validates_length_of( :title, :maximum=>5, :too_long=> N_("%{fn} hoo %d"), :if => :condition_is_true )
@@ -1576,8 +2159,8 @@ class ValidationsTest < Test::Unit::TestCase
     t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
     assert !t.valid?
     assert t.errors.on(:title)
-    assert_equal "タイトルふー5", t.errors.on("title")
     assert_equal "タイトルふー5", t.errors.full_messages[0]
+    assert_equal "タイトルふー5", t.errors.on("title")
 
     GetText.set_locale_all "en"
     t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
@@ -1587,6 +2170,54 @@ class ValidationsTest < Test::Unit::TestCase
     assert_equal "Title hoo 5", t.errors.on("title")
   end
 
+=begin
+  Don't need this
+  def test_unless_validation_using_block_true
+  end
+  def test_if_validation_using_block_false
+  end
+=end
+
+  def test_unless_validation_using_block_false_fn
+    # When the block returns false
+    Topic.validates_length_of( :title, :maximum=>5, :too_long=>N_("%{fn} hoo %d"),
+      :unless => Proc.new { |r| r.title != "uhohuhoh"} )
+
+    GetText.set_locale_all "ja_JP.UTF-8"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "タイトルふー5", t.errors.full_messages[0]
+    assert_equal "タイトルふー5", t.errors.on("title")
+
+    GetText.set_locale_all "en"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "Title hoo 5", t.errors.full_messages[0]
+    assert_equal "Title hoo 5", t.errors["title"]
+  end
+
+  def test_unless_validation_using_block_false
+    # When the block returns false
+    Topic.validates_length_of( :title, :maximum=>5, :too_long=>N_("hoo %d"),
+      :unless => Proc.new { |r| r.title != "uhohuhoh"} )
+
+    GetText.set_locale_all "ja_JP.UTF-8"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "タイトル ふー5", t.errors.full_messages[0]
+    assert_equal "ふー5", t.errors.on("title")
+
+    GetText.set_locale_all "en"
+    t = Topic.create("title" => "uhohuhoh", "content" => "whatever")
+    assert !t.valid?
+    assert t.errors.on(:title)
+    assert_equal "Title hoo 5", t.errors.full_messages[0]
+    assert_equal "hoo 5", t.errors["title"]
+  end
+
   def test_conditional_validation_using_method_true
     # When the method returns true
     Topic.validates_length_of( :title, :maximum=>5, :too_long=> N_("hoo %d"), :if => :condition_is_true )
@@ -1732,6 +2363,12 @@ class ValidationsTest < Test::Unit::TestCase
     end
   end
 
+=begin
+  #Don't need this
+  def test_validation_with_if_as_string
+  end
+=end
+
   def test_default_validates_numericality_of
     GetText.set_locale_all "ja_JP.UTF-8"
     Topic.validates_numericality_of :approved
@@ -1742,7 +2379,7 @@ class ValidationsTest < Test::Unit::TestCase
     topic = Topic.create("title" => "numeric test", "content" => "whatever", "approved" => "aaa")
     assert_equal "Approved is not a number", topic.errors.on(:approved)
     assert_equal "Approved is not a number", topic.errors.full_messages[0]
-   end
+  end
 
   def test_inherited_messages
     GetText.set_locale_all "ja_JP.UTF-8"
@@ -1758,29 +2395,35 @@ class ValidationsTest < Test::Unit::TestCase
 
     t = Reply.create
     assert !t.save
-    assert_equal "Replyタイトル 空です。", t.errors.full_messages[0]
-    assert_equal "Reply内容 空です。", t.errors.full_messages[1]
-    assert_equal "空です。", t.errors.on(:title)
-    assert_equal "空です。", t.errors.on(:content)
+    assert_equal "Replyタイトルを入力してください。", t.errors.full_messages[0]
+    assert_equal "Replyタイトル 空です。", t.errors.full_messages[1]
+    assert_equal "Reply内容を入力してください。", t.errors.full_messages[2]
+    assert_equal "Reply内容 空です。", t.errors.full_messages[3]
+    assert_equal ["Replyタイトルを入力してください。","空です。"], t.errors.on(:title)
+    assert_equal ["Reply内容を入力してください。", "空です。"], t.errors.on(:content)
 
     t = Reply.create
     t.title = "Wrong Create"
     assert !t.save
     assert_equal "Replyタイトル が不正に生成されました。", t.errors.full_messages[0]
-    assert_equal "Reply内容 空です。", t.errors.full_messages[1]
+    assert_equal "Reply内容を入力してください。", t.errors.full_messages[1]
+    assert_equal "Reply内容 空です。", t.errors.full_messages[2]
 
     t = SillyReply.create
     assert !t.save
-    assert_equal "Sillyタイトル 空です。", t.errors.full_messages[0]
-    assert_equal "Silly内容 空です。", t.errors.full_messages[1]
-    assert_equal "空です。", t.errors.on(:title)
-    assert_equal "空です。", t.errors.on(:content)
+    assert_equal "Sillyタイトルを入力してください。", t.errors.full_messages[0]
+    assert_equal "Sillyタイトル 空です。", t.errors.full_messages[1]
+    assert_equal "Silly内容を入力してください。", t.errors.full_messages[2]
+    assert_equal "Silly内容 空です。", t.errors.full_messages[3]
+    assert_equal ["Sillyタイトルを入力してください。","空です。"], t.errors.on(:title)
+    assert_equal ["Silly内容を入力してください。","空です。"], t.errors.on(:content)
 
     t = SillyReply.create
     t.title = "Wrong Create"
     assert !t.save
     assert_equal "Sillyタイトル が不正に生成されました。", t.errors.full_messages[0]
-    assert_equal "Silly内容 空です。", t.errors.full_messages[1]
+    assert_equal "Silly内容を入力してください。", t.errors.full_messages[1]
+    assert_equal "Silly内容 空です。", t.errors.full_messages[2]
   end
 
   def test_original_model_with_validation
@@ -1800,3 +2443,164 @@ class ValidationsTest < Test::Unit::TestCase
   end
 end
 
+class ValidatesNumericalityTest < Test::Unit::TestCase
+  include GetText
+
+  NIL = [nil]
+  BLANK = ["", " ", " \t \r \n"]
+  BIGDECIMAL_STRINGS = %w(12345678901234567890.1234567890) # 30 significent digits
+  FLOAT_STRINGS = %w(0.0 +0.0 -0.0 10.0 10.5 -10.5 -0.0001 -090.1 90.1e1 -90.1e5 -90.1e-5 90e-5)
+  INTEGER_STRINGS = %w(0 +0 -0 10 +10 -10 0090 -090)
+  FLOATS = [0.0, 10.0, 10.5, -10.5, -0.0001] + FLOAT_STRINGS
+  INTEGERS = [0, 10, -10] + INTEGER_STRINGS
+  BIGDECIMAL = BIGDECIMAL_STRINGS.collect! { |bd| BigDecimal.new(bd) }
+  JUNK = ["not a number", "42 not a number", "0xdeadbeef", "00-1", "--3", "+-3", "+3-1", "-+019.0", "12.12.13.12", "123\nnot a number"]
+
+  def setup
+    bindtextdomain_to(Topic, "active_record")
+
+    Topic.instance_variable_set("@validate_callbacks", ActiveSupport::Callbacks::CallbackChain.new)
+    Topic.instance_variable_set("@validate_on_create_callbacks", ActiveSupport::Callbacks::CallbackChain.new)
+    Topic.instance_variable_set("@validate_on_update_callbacks", ActiveSupport::Callbacks::CallbackChain.new)
+  end
+
+  def test_default_validates_numericality_of
+    Topic.validates_numericality_of :approved
+
+    invalid!(NIL + BLANK + JUNK)
+#    valid!(FLOATS + INTEGERS + BIGDECIMAL)
+  end
+
+  def test_validates_numericality_of_with_nil_allowed
+    Topic.validates_numericality_of :approved, :allow_nil => true
+
+    invalid!(BLANK + JUNK)
+ #   valid!(NIL + FLOATS + INTEGERS + BIGDECIMAL)
+  end
+
+  def test_validates_numericality_of_with_integer_only
+    Topic.validates_numericality_of :approved, :only_integer => true
+
+    invalid!(NIL + BLANK + JUNK + FLOATS + BIGDECIMAL)
+ #   valid!(INTEGERS)
+  end
+
+  def test_validates_numericality_of_with_integer_only_and_nil_allowed
+    Topic.validates_numericality_of :approved, :only_integer => true, :allow_nil => true
+
+    invalid!(BLANK + JUNK + FLOATS + BIGDECIMAL)
+ #   valid!(NIL + INTEGERS)
+  end
+
+  def test_validates_numericality_with_greater_than
+    Topic.validates_numericality_of :approved, :greater_than => 10
+
+    invalid!([-10, 10], 'Approved must be greater than 10',
+             '承認は10より大きい値にしてください。')
+#    valid!([11])
+  end
+
+  def test_validates_numericality_with_greater_than_or_equal
+    Topic.validates_numericality_of :approved, :greater_than_or_equal_to => 10
+
+    invalid!([-9, 9], 'Approved must be greater than or equal to 10',
+             "承認は10以上の値にしてください。")
+ #   valid!([10])
+  end
+
+  def test_validates_numericality_with_equal_to
+    Topic.validates_numericality_of :approved, :equal_to => 10
+
+    invalid!([-10, 11], 'Approved must be equal to 10',
+             "承認は10にしてください。")
+ #   valid!([10])
+  end
+
+  def test_validates_numericality_with_less_than
+    Topic.validates_numericality_of :approved, :less_than => 10
+
+    invalid!([10], 'Approved must be less than 10',
+             "承認は10より小さい値にしてください。")
+ #   valid!([-9, 9])
+  end
+
+  def test_validates_numericality_with_less_than_or_equal_to
+    Topic.validates_numericality_of :approved, :less_than_or_equal_to => 10
+
+    invalid!([11], 'Approved must be less than or equal to 10')
+ #   valid!([-10, 10])
+  end
+
+  def test_validates_numericality_with_odd
+    Topic.validates_numericality_of :approved, :odd => true
+
+    invalid!([-2, 2], 'Approved must be odd',
+             "承認は奇数にしてください。")
+ #   valid!([-1, 1])
+  end
+
+  def test_validates_numericality_with_even
+    Topic.validates_numericality_of :approved, :even => true
+
+    invalid!([-1, 1], 'Approved must be even',
+             "承認は偶数にしてください。")
+ #   valid!([-2, 2])
+  end
+
+  def test_validates_numericality_with_greater_than_less_than_and_even
+    Topic.validates_numericality_of :approved, :greater_than => 1, :less_than => 4, :even => true
+
+    invalid!([1, 3, 4])
+ #   valid!([2])
+  end
+
+  def test_validates_numericality_with_numeric_message
+    Topic.validates_numericality_of :approved, :less_than => 4, :message => N_("smaller than %d")
+    topic = Topic.new("title" => "numeric test", "approved" => 10)
+
+    assert !topic.valid?
+    assert_equal "Approved smaller than 4", topic.errors.full_messages[0]
+    assert_equal "smaller than 4", topic.errors.on(:approved)
+
+    Topic.validates_numericality_of :approved, :greater_than => 4, :message => N_("greater than %d")
+    topic = Topic.new("title" => "numeric test", "approved" => 1)
+
+    assert !topic.valid?
+    assert_equal "Approved greater than 4", topic.errors.full_messages[0]
+    assert_equal "greater than 4", topic.errors.on(:approved)
+  end
+
+  private
+    def invalid!(values, error=nil, errorj=nil)
+      with_each_topic_approved_value(values) do |topic, value|
+        GetText.set_locale_all "ja"
+        assert !topic.valid?, "#{value.inspect} not rejected as a number"
+        assert topic.errors.on(:approved)
+        assert_equal errorj, topic.errors.full_messages[0] if errorj
+        assert_equal errorj, topic.errors.on(:approved) if errorj
+      end
+
+      with_each_topic_approved_value(values) do |topic, value|
+        GetText.set_locale_all "en"
+        assert !topic.valid?, "#{value.inspect} not rejected as a number"
+        assert topic.errors.on(:approved)
+        assert_equal error, topic.errors.full_messages[0] if error
+        assert_equal error, topic.errors.on(:approved) if error
+      end
+    end
+=begin
+    # Don't need this
+    def valid!(values)
+      with_each_topic_approved_value(values) do |topic, value|
+        assert topic.valid?, "#{value.inspect} not accepted as a number"
+      end
+    end
+=end
+    def with_each_topic_approved_value(values)
+      topic = Topic.new("title" => "numeric test", "content" => "whatever")
+      values.each do |value|
+        topic.approved = value
+        yield topic, value
+      end
+    end
+end
